@@ -23,6 +23,7 @@
  */
 
 #include <string.h>
+#include <ctype.h>
 
 #include <atk/atk.h>
 #include <droute/droute.h>
@@ -55,15 +56,16 @@ typedef struct _SpiReentrantCallClosure
 static void
 switch_main_context (GMainContext *cnx)
 {
-#ifndef DISABLE_P2P
 /* This code won't work on dbus-glib earlier than 0.9.0 because of FDO#30574 */
+  if (spi_global_app_data->app_bus_addr [0] == '\0')
+    return;
+
   GList *list;
 
   dbus_server_setup_with_g_main (spi_global_app_data->server, cnx);
   dbus_connection_setup_with_g_main (spi_global_app_data->bus, cnx);
   for (list = spi_global_app_data->direct_connections; list; list = list->next)
     dbus_connection_setup_with_g_main (list->data, cnx);
-#endif
 }
 
 static void
@@ -396,8 +398,7 @@ emit_event (AtkObject  *obj,
             void (*append_variant) (DBusMessageIter *, const char *, const void *))
 {
   DBusConnection *bus = spi_global_app_data->bus;
-  const char *path =  spi_register_object_to_path (spi_global_register,
-                                                   G_OBJECT (obj));
+  const char *path;
 
   gchar *cname, *t;
   DBusMessage *sig;
@@ -411,6 +412,8 @@ emit_event (AtkObject  *obj,
   if (!signal_is_needed (klass, major, minor))
     return;
 
+  path =  spi_register_object_to_path (spi_global_register, G_OBJECT (obj));
+
   /*
    * This is very annoying, but as '-' isn't a legal signal
    * name in D-Bus (Why not??!?) The names need converting
@@ -434,6 +437,7 @@ emit_event (AtkObject  *obj,
     spi_object_lease_if_needed (G_OBJECT (obj));
 
   g_free(cname);
+  g_free (path);
 }
 
 /*---------------------------------------------------------------------------*/
